﻿Option Strict On

Imports Nist.Logging
Imports System.Xml

<ServiceModel.ServiceBehavior(Namespace:="http://docs.oasis-open.org/bias/ns/bias-1.0/")>
Public Class BIASService
    Implements BIAS_v1

    Public Sub New()
    End Sub

    Public Function QueryCapabilities(ByVal QueryCapabilitiesRequest As QueryCapabilitiesRequest) As QueryCapabilitiesResponsePackage Implements BIAS_v1.QueryCapabilities
        Debug("QueryCapabilities()")
        Dim Response As New QueryCapabilitiesResponsePackage()

        Response.ResponseStatus = New ResponseStatus
        Response.ResponseStatus.Return = 0

        Response.CapabilityList = New CapabilityListType()
        Dim cap As New CapabilityType()
        cap.CapabilityName = CapabilityName.AggregateInputDataOptional
        Response.CapabilityList.Add(cap)
        cap = New CapabilityType()
        cap.CapabilityName = CapabilityName.AggregateInputDataRequired
        Response.CapabilityList.Add(cap)
        cap = New CapabilityType()
        cap.CapabilityName = CapabilityName.AggregateProcessingOption
        Response.CapabilityList.Add(cap)
        cap = New CapabilityType()
        cap.CapabilityName = CapabilityName.AggregateReturnData
        Response.CapabilityList.Add(cap)
        cap = New CapabilityType()
        cap.CapabilityName = CapabilityName.AggregateServiceDescription
        Response.CapabilityList.Add(cap)
        cap = New CapabilityType()
        cap.CapabilityName = CapabilityName.BiographicDataSet
        Response.CapabilityList.Add(cap)
        cap = New CapabilityType()
        cap.CapabilityName = CapabilityName.CBEFFPatronFormat
        Response.CapabilityList.Add(cap)
        cap = New CapabilityType()
        cap.CapabilityName = CapabilityName.ConformanceClass
        Response.CapabilityList.Add(cap)
        cap = New CapabilityType()
        cap.CapabilityName = CapabilityName.IdentityModel
        Response.CapabilityList.Add(cap)
        cap = New CapabilityType()
        cap.CapabilityName = CapabilityName.SupportedBiometric
        Response.CapabilityList.Add(cap)

        Return Response
    End Function

    Public Function Enroll(ByVal EnrollRequest As EnrollRequest) As EnrollResponsePackage Implements BIAS_v1.Enroll
        Debug("Enroll()")
        Info("Processing new enrollment")

        Dim Result As New EnrollResponsePackage()
        Result.ResponseStatus = New ResponseStatus

        If Gallery.Subjects.ContainsKey(EnrollRequest.InputData.Nodes(1).InnerText) Then
            Info("Subject is already enrolled")
            Result.ResponseStatus.Message = "Subject with the specified identification key is already enrolled in this system."
            Result.ResponseStatus.Return = 1
        Else
            Dim Enrollee As New Subject()

            Enrollee.Name = EnrollRequest.InputData.Nodes(0).InnerText
            Enrollee.MD5 = EnrollRequest.InputData.Nodes(1).InnerText
            Enrollee.DataType = EnrollRequest.InputData.Nodes(2).InnerText
            Enrollee.Data = EnrollRequest.InputData.Nodes(3).InnerText

            Gallery.Subjects.Add(EnrollRequest.InputData.Nodes(1).InnerText, Enrollee)
            Result.ResponseStatus.Return = 0
            Info("Enrollment Successful")
        End If

        Return Result
    End Function

    Public Function Verify(ByVal VerifyRequest As VerifyRequest) As VerifyResponsePackage Implements BIAS_v1.Verify
        Debug("Verify()")
        Info("Processing verification")

        Dim IdentityClaim As String = VerifyRequest.Identity.IdentityClaim
        Debug(" - Identity Claim = {0}", IdentityClaim)

        Dim Result As New VerifyResponsePackage()
        Result.ResponseStatus = New ResponseStatus
        Result.ResponseStatus.Return = 0
        Result.Match = False
        Result.Score = 0

        If Gallery.Subjects.ContainsKey(IdentityClaim) Then
            Dim Target As Subject = Gallery.Subjects(IdentityClaim)
            If Target.MD5.Equals(VerifyRequest.InputData.Nodes(1).InnerText, StringComparison.CurrentCultureIgnoreCase) AndAlso Target.Data.Equals(VerifyRequest.InputData.Nodes(3).InnerText, StringComparison.CurrentCultureIgnoreCase) Then
                Info("Verification successful; target and subject match")
                Result.Match = True
                Result.Score = 100
            Else
                Info("Verification failed; target and subject do not match")
                Result.ResponseStatus.Message = "Verification failed; target and subject do not match"
                Result.Match = False
                Result.Score = 0
            End If
        Else
            Info("Verification failed; claimed identity is not enrolled")
            Result.ResponseStatus.Message = "Verification failed; claimed identity is not enrolled"
            Result.Match = False
            Result.Score = 0
        End If

        Return Result
    End Function

    Public Function RetrieveInformation(ByVal RetrieveInformationRequest As RetrieveInformationRequest) As RetrieveInformationResponsePackage Implements BIAS_v1.RetrieveInformation
        Debug("RetrieveInformation()")
        Info("Retrieving information")

        Dim Result As New RetrieveInformationResponsePackage()
        Result.ResponseStatus = New ResponseStatus()

        If Gallery.Subjects.ContainsKey(RetrieveInformationRequest.Identity.SubjectID) Then
            Info("Target is enrolled; returning information")
            Result.ResponseStatus.Return = 0
            Result.ReturnData = New InformationType()

            Dim ReturnData = <SubjectInformation>
                                 <Name>John Doe</Name>
                                 <MD5>d41d8cd98f00b204e9800998ecf8427e</MD5>
                                 <DataType>image/jpeg</DataType>
                                 <Data>cXdlcnRhc3Jnc2RmZ2Flc2Zna3UzMjU3ODk0NTYgb2ZnZHUgamt4bmggaSBlNDA4NnJzdGlvaCB4a2NqZ24gbzg1NCA=</Data>
                             </SubjectInformation>

            Result.ReturnData = New InformationType
            Result.ReturnData.Nodes = New XmlNode() {New XmlDocument().ReadNode(ReturnData.CreateReader())}
        Else
            Info("Target is not enrolled; no information will be returned")
            Result.ResponseStatus.Return = 1
        End If

        Return Result
    End Function

    Public Function Identify(ByVal IdentifyRequest As IdentifyRequest) As IdentifyResponsePackage Implements BIAS_v1.Identify
        Debug("Identify()")
        Info("Identifying subject")

        Dim Result As New IdentifyResponsePackage()
        Result.ResponseStatus = New ResponseStatus()

        Dim Data As String = IdentifyRequest.InputData.Nodes(1).InnerText
        Dim SubjectPair As KeyValuePair(Of String, Subject) = Nothing
        Dim Identified As Boolean = False

        For Each SubjectPair In Gallery.Subjects
            If SubjectPair.Value.Data.Equals(Data, StringComparison.CurrentCultureIgnoreCase) Then
                Info("Subject identified as {0} [{1}]", SubjectPair.Value.Name, SubjectPair.Key)
                Identified = True
                Exit For
            End If
        Next

        If Identified Then
            Dim ReturnData = <SubjectInformation>
                                 <Name><%= SubjectPair.Value.Name %></Name>
                                 <MD5><%= SubjectPair.Value.MD5 %></MD5>
                             </SubjectInformation>

            Result.ReturnData = New InformationType
            Result.ReturnData.Nodes = New XmlNode() {New XmlDocument().ReadNode(ReturnData.CreateReader())}
            Result.ResponseStatus.Return = 0
        Else
            'NOT identified
            Result.ResponseStatus.Return = 1
        End If

        Return Result
    End Function

    'Asynchronous
    Public Function GetEnrollResults(ByVal GetEnrollResultsRequest As GetEnrollResultsRequest) As GetEnrollResultsResponsePackage Implements BIAS_v1.GetEnrollResults
        Debug("GetEnrollResults()")
        Return Nothing
    End Function

    'Asynchronous
    Public Function GetIdentifyResults(ByVal GetIdentifyResultsRequest As GetIdentifyResultsRequest) As GetIdentifyResultsResponsePackage Implements BIAS_v1.GetIdentifyResults
        Debug("GetIdentifyResults()")
        Return Nothing
    End Function

    'Asynchronous
    Public Function GetVerifyResults(ByVal GetVerifyResultsRequest As GetVerifyResultsRequest) As GetVerifyResultsResponsePackage Implements BIAS_v1.GetVerifyResults
        Debug("GetVerifyResults()")
        Return Nothing
    End Function


    ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' '
    ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' ' '

    Public Function AddSubjectToGallery(ByVal AddSubjectToGalleryRequest As AddSubjectToGalleryRequest) As AddSubjectToGalleryResponsePackage Implements BIAS_v1.AddSubjectToGallery
        Debug("AddSubjectToGallery()")
        Return Nothing
    End Function

    Public Function CheckQuality(ByVal CheckQualityRequest As CheckQualityRequest) As CheckQualityResponsePackage Implements BIAS_v1.CheckQuality
        Debug("CheckQuality()")
        Return Nothing
    End Function

    Public Function ClassifyBiometricData(ByVal ClassifyBiometricDataRequest As ClassifyBiometricDataRequest) As ClassifyBiometricDataResponsePackage Implements BIAS_v1.ClassifyBiometricData
        Debug("ClassifyBiometricData()")
        Return Nothing
    End Function

    Public Function CreateSubject(ByVal CreateSubjectRequest As CreateSubjectRequest) As CreateSubjectResponsePackage Implements BIAS_v1.CreateSubject
        Debug("CreateSubject()")
        Return Nothing
    End Function

    Public Function DeleteBiographicData(ByVal DeleteBiographicDataRequest As DeleteBiographicDataRequest) As DeleteBiographicDataResponsePackage Implements BIAS_v1.DeleteBiographicData
        Debug("DeleteBiographicData()")
        Return Nothing
    End Function

    Public Function DeleteBiometricData(ByVal DeleteBiometricDataRequest As DeleteBiometricDataRequest) As DeleteBiometricDataResponsePackage Implements BIAS_v1.DeleteBiometricData
        Debug("DeleteBiometricData()")
        Return Nothing
    End Function

    Public Function DeleteSubject(ByVal DeleteSubjectRequest As DeleteSubjectRequest) As DeleteSubjectResponsePackage Implements BIAS_v1.DeleteSubject
        Debug("DeleteSubject()")
        Return Nothing
    End Function

    Public Function DeleteSubjectFromGallery(ByVal DeleteSubjectFromGalleryRequest As DeleteSubjectFromGalleryRequest) As DeleteSubjectFromGalleryResponsePackage Implements BIAS_v1.DeleteSubjectFromGallery
        Debug("DeleteSubjectFromGallery()")
        Return Nothing
    End Function

    Public Function GetIdentifySubjectResults(ByVal GetIdentifySubjectResultsRequest As GetIdentifySubjectResultsRequest) As GetIdentifySubjectResultsResponsePackage Implements BIAS_v1.GetIdentifySubjectResults
        Debug("GetIdentifySubjectResults()")
        Return Nothing
    End Function

    Public Function IdentifySubject(ByVal IdentifySubjectRequest As IdentifySubjectRequest) As IdentifySubjectResponsePackage Implements BIAS_v1.IdentifySubject
        Debug("IdentifySubject()")
        Return Nothing
    End Function

    Public Function ListBiographicData(ByVal ListBiographicDataRequest As ListBiographicDataRequest) As ListBiographicDataResponsePackage Implements BIAS_v1.ListBiographicData
        Debug("ListBiographicData()")
        Return Nothing
    End Function

    Public Function ListBiometricData(ByVal ListBiometricDataRequest As ListBiometricDataRequest) As ListBiometricDataResponsePackage Implements BIAS_v1.ListBiometricData
        Debug("ListBiometricData()")
        Return Nothing
    End Function

    Public Function PerformFusion(ByVal PerformFusionRequest As PerformFusionRequest) As PerformFusionResponsePackage Implements BIAS_v1.PerformFusion
        Debug("PerformFusion()")
        Return Nothing
    End Function

    Public Function RetrieveBiographicInformation(ByVal RetrieveBiographicInformationRequest As RetrieveBiographicInformationRequest) As RetrieveBiographicInformationResponsePackage Implements BIAS_v1.RetrieveBiographicInformation
        Debug("RetrieveBiographicInformation()")
        Return Nothing
    End Function

    Public Function RetrieveBiometricInformation(ByVal RetrieveBiometricInformationRequest As RetrieveBiometricInformationRequest) As RetrieveBiometricInformationResponsePackage Implements BIAS_v1.RetrieveBiometricInformation
        Debug("RetrieveBiometricInformation()")
        Return Nothing
    End Function

    Public Function SetBiographicData(ByVal SetBiographicDataRequest As SetBiographicDataRequest) As SetBiographicDataResponsePackage Implements BIAS_v1.SetBiographicData
        Debug("SetBiographicData()")
        Return Nothing
    End Function

    Public Function SetBiometricData(ByVal SetBiometricDataRequest As SetBiometricDataRequest) As SetBiometricDataResponsePackage Implements BIAS_v1.SetBiometricData
        Debug("SetBiometricData()")
        Return Nothing
    End Function

    Public Function TransformBiometricData(ByVal TransformBiometricDataRequest As TransformBiometricDataRequest) As TransformBiometricDataResponsePackage Implements BIAS_v1.TransformBiometricData
        Debug("TransformBiometricData()")
        Return Nothing
    End Function

    Public Function UpdateBiographicData(ByVal UpdateBiographicDataRequest As UpdateBiographicDataRequest) As UpdateBiographicDataResponsePackage Implements BIAS_v1.UpdateBiographicData
        Debug("UpdateBiographicData()")
        Return Nothing
    End Function

    Public Function UpdateBiometricData(ByVal UpdateBiometricDataRequest As UpdateBiometricDataRequest) As UpdateBiometricDataResponsePackage Implements BIAS_v1.UpdateBiometricData
        Debug("UpdateBiometricData()")
        Return Nothing
    End Function

    Public Function VerifySubject(ByVal VerifySubjectRequest As VerifySubjectRequest) As VerifySubjectResponsePackage Implements BIAS_v1.VerifySubject
        Debug("VerifySubject()")
        Return Nothing
    End Function
End Class